local addon, private = ...
local mod = LibStub("AceAddon-3.0"):NewAddon(addon, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local ArtifactUI          = _G.C_ArtifactUI

function getSocketSlotID()
  local class = select(2, UnitClass("player"))
  if (class == "PALADIN" and GetSpecialization() == 2) or (class == "WARRIOR" and GetSpecialization() == 3) then
    return 17
  else
    return 16
  end
end

function mod:Update()
  self:BuildAllLines()
  self.CopyFrame:Show()
end

function mod:copySlotsFromEquipped()
  for _, frame in ipairs(self.frameSlots) do
    if frame.info.slot_id and not frame.info.artifact_gem_id then
      frame.setItem(GetInventoryItemLink('player', frame.info.slot_id))
    else
      frame.setItem(nil)
    end

    if frame.info.artifact_gem_id then
      local _, gemLink = GetItemGem(GetInventoryItemLink('player', frame.info.slot_id), frame.info.artifact_gem_id)
      frame.setItem(gemLink)
    end
  end
  self:Update()
end

function mod:OnInitialize()
  mod:RegisterChatCommand('sim', self.loadWindow)
end

function mod:BuildTalentString()
  local talentInfo = {}
  local maxTiers = 7
  local maxColumns = 3
  for tier = 1, maxTiers do
    for column = 1, maxColumns do
      local talentID, name, iconTexture, selected, available = GetTalentInfo(tier, column, GetActiveSpecGroup())
      if selected then
        talentInfo[tier] = column
      end
    end
  end

  local str = 'talents='
  for i = 1, maxTiers do
    if talentInfo[i] then
      str = str .. talentInfo[i]
    else
      str = str .. '0'
    end
  end

  return str
end

function mod:BuildArtifactString()
  local item_id = select(1, ArtifactUI.GetArtifactInfo())
  if item_id == nil or item_id == 0 then
    return nil
  end

  local artifact_id = self.ARTIFACT_TABLE[item_id]
  if artifact_id == nil then
    return nil
  end

  -- Note, relics are handled by the item string
  local str = 'artifact=' .. artifact_id .. ':0:0:0:0'

  local powers = ArtifactUI.GetPowers()
  for i = 1, #powers do
    local power_id = powers[i]
    local _, _, currentRank, _, bonusRanks = ArtifactUI.GetPowerInfo(power_id)
    if currentRank > 0 and currentRank - bonusRanks > 0 then
      str = str .. ':' .. power_id .. ':' .. (currentRank - bonusRanks)
    end
  end

  return str
end

function mod:BuildSimcLine(itemLink)
  local itemName = GetItemInfo(itemLink)
  local itemString = string.match(itemLink, "item:([%-?%d:]+)")
  local itemSplit = {}
  local simcItemOptions = {}
  local slug_name = gsub(gsub(string.lower(itemName), " ", "_"), "[^%a_]", "")
  tinsert(simcItemOptions, slug_name)

  -- Split data into a table
  for v in string.gmatch(itemString, "(%d*:?)") do
    if v == ":" then
      itemSplit[#itemSplit + 1] = 0
    else
      itemSplit[#itemSplit + 1] = string.gsub(v, ':', '')
    end
  end

  -- Item id
  local itemId = itemSplit[self.OFFSETS.ITEM_ID]
  simcItemOptions[#simcItemOptions + 1] = 'id=' .. itemId

    -- Enchant
  if tonumber(itemSplit[self.OFFSETS.ENCHANT_ID]) > 0 then
    simcItemOptions[#simcItemOptions + 1] = 'enchant_id=' .. itemSplit[self.OFFSETS.ENCHANT_ID]
  end

  -- New style item suffix, old suffix style not supported
  if tonumber(itemSplit[self.OFFSETS.SUFFIX_ID]) ~= 0 then
    simcItemOptions[#simcItemOptions + 1] = 'suffix=' .. itemSplit[self.OFFSETS.SUFFIX_ID]
  end

  local flags = tonumber(itemSplit[self.OFFSETS.FLAGS])
  local bonuses = {}

  for index=1, tonumber(itemSplit[self.OFFSETS.BONUS_ID]) do
    bonuses[#bonuses + 1] = itemSplit[self.OFFSETS.BONUS_ID + index]
  end

  if #bonuses > 0 then
    simcItemOptions[#simcItemOptions + 1] = 'bonus_id=' .. table.concat(bonuses, '/')
  end

  local rest_offset = self.OFFSETS.BONUS_ID + #bonuses + 1

  -- Upgrade level
  if bit.band(flags, 4) == 4 then
    local upgrade_id = tonumber(itemSplit[rest_offset])
    if self.UPGRADE_TABLE[upgrade_id] ~= nil and self.UPGRADE_TABLE[upgrade_id] > 0 then
      simcItemOptions[#simcItemOptions + 1] = 'upgrade=' .. self.UPGRADE_TABLE[upgrade_id]
    end
    rest_offset = rest_offset + 1
  end

  -- Artifacts use this
  if bit.band(flags, 256) == 256 then
    rest_offset = rest_offset + 1 -- An unknown field
    local relic_str = ''
    while rest_offset < #itemSplit do
      local n_bonus_ids = tonumber(itemSplit[rest_offset])
      rest_offset = rest_offset + 1

      if n_bonus_ids == 0 then
        relic_str = relic_str .. 0
      else
        for rbid = 1, n_bonus_ids do
          relic_str = relic_str .. itemSplit[rest_offset]
          if rbid < n_bonus_ids then
            relic_str = relic_str .. ':'
          end
          rest_offset = rest_offset + 1
        end
      end

      if rest_offset < #itemSplit then
        relic_str = relic_str .. '/'
      end
    end

    if relic_str ~= '' then
      simcItemOptions[#simcItemOptions + 1] = 'relic_id=' .. relic_str
    end
  end

  -- Some leveling quest items seem to use this, it'll include the drop level of the item
  if bit.band(flags, 512) == 512 then
    simcItemOptions[#simcItemOptions + 1] = 'drop_level=' .. itemSplit[rest_offset]
    rest_offset = rest_offset + 1
  end

  -- Gems
  local gems = {}

  -- Get sockets for artifacts
  local itemType = select(9, GetItemInfo(itemLink))
  if ((itemType == "INVTYPE_WEAPONMAINHAND" and getSocketSlotID() == 16) or (itemType == "INVTYPE_SHIELD" and getSocketSlotID() == 17)) then
    for _, frame in ipairs(mod.frameSlots) do
      if frame.itemLink and frame.info.artifact_gem_id then
        local gemLink = frame.itemLink
        if gemLink then
          local gemDetail = string.match(gemLink, "item[%-?%d:]+")
          gems[#gems + 1] = string.match(gemDetail, "item:(%d+):" )
        elseif flags == 256 then
          gems[#gems + 1] = "0"
        end
      end
    end

  -- Sockets for non-artifacts
  else
    for i=1, 4 do -- hardcoded here to just grab all 4 sockets
      local _, gemLink = GetItemGem(itemLink, i)
      if gemLink then
        local gemDetail = string.match(gemLink, "item[%-?%d:]+")
        gems[#gems + 1] = string.match(gemDetail, "item:(%d+):" )
      elseif flags == 256 then
        gems[#gems + 1] = "0"
      end
    end
  end


  if #gems > 0 then
    simcItemOptions[#simcItemOptions + 1] = 'gem_id=' .. table.concat(gems, '/')
  end

  return table.concat(simcItemOptions, ',')
end

function mod:BuildAllLines()
  local text = "# Append this to your SimC profile to compare your current loadout against a proposed loadout\n"
  text = text .. 'copy="' .. UnitName("player") .. ' - ' .. select(2, GetSpecializationInfo(GetSpecialization())).. '"\n'
  text = text .. self:BuildTalentString() .. "\n" .. self:BuildArtifactString() .. "\n"
  for _, frame in ipairs(mod.frameSlots) do
    if frame.info.slot_id and not frame.info.artifact_gem_id and frame.itemLink then
      local line = self:BuildSimcLine(frame.itemLink)
      local name, itemLink, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice, classId, subClassId = GetItemInfo(frame.itemLink)
      text = text .. mod.SIMC_SLOT_NAMES[equipSlot] .. "=" .. line .. "\n"
    end
  end
  mod.editBox:SetText(text)
end